package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.*;
import com.usky.iot.mapper.BaseAlarmMapper;
import com.usky.iot.service.*;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.service.config.websocket.WebSocket;
import com.usky.iot.service.vo.BaseAlarmListVO;
import com.usky.iot.service.vo.BaseAlarmRequestVO;
import com.usky.iot.service.vo.BaseAlarmResponeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 统一告警表 服务实现类
 * </p>
 *
 * @author han
 * @since 2023-07-13
 */
@Service
public class BaseAlarmServiceImpl extends AbstractCrudService<BaseAlarmMapper, BaseAlarm> implements BaseAlarmService {

    @Autowired
    private DmpProductInfoService dmpProductInfoService;

    @Autowired
    private DmpDeviceInfoService dmpDeviceInfoService;

    @Autowired
    private BaseFacilityDeviceService baseFacilityDeviceService;

    @Autowired
    private WebSocket webSocket;

    @Override
    public CommonPage<BaseAlarmResponeVO> statistic(BaseAlarmRequestVO baseAlarmRequestVO){

        List<BaseAlarmResponeVO> list = new ArrayList<>();

        LambdaQueryWrapper<DmpProductInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpProductInfo::getDeleteFlag,0)
                .like(StringUtils.isNotBlank(baseAlarmRequestVO.getProductName()),DmpProductInfo::getProductName,baseAlarmRequestVO.getProductName())
                .eq(DmpProductInfo::getTenantId,SecurityUtils.getTenantId());
        List<DmpProductInfo> list1 = dmpProductInfoService.list(queryWrapper);
        if(CollectionUtils.isNotEmpty(list1)){
            for(int i=0;i<list1.size();i++){
                BaseAlarmResponeVO responeVO = new BaseAlarmResponeVO();
                responeVO.setProductCode(list1.get(i).getProductCode());
                responeVO.setProductName(list1.get(i).getProductName());
                responeVO.setDeviceType(list1.get(i).getDeviceType());

                //未处理
                int untreated = 0;
                untreated = this.count(Wrappers.<BaseAlarm>query().lambda().eq(BaseAlarm::getHandleStatus,0).eq(BaseAlarm::getProductCode,list1.get(i).getProductCode()).eq(BaseAlarm::getTenantId,SecurityUtils.getTenantId()));
                //已处理
                int processed = 0;
                processed = this.count(Wrappers.<BaseAlarm>query().lambda().eq(BaseAlarm::getHandleStatus,1).eq(BaseAlarm::getProductCode,list1.get(i).getProductCode()).eq(BaseAlarm::getTenantId,SecurityUtils.getTenantId()));
                //总数
                int count = untreated+processed;
                //处置率
                float processRate = 0;
                if(count != 0){
                    processRate = ((float) processed/count)*100;
                }

                responeVO.setTotal(count);
                responeVO.setUntreated(untreated);
                responeVO.setProcessed(processed);
                responeVO.setProcessRate(processRate);

                //设备总数
                int deviceCount = 0;
                List<Integer> serviceStatusList = new ArrayList<>();
                serviceStatusList.add(1);
                serviceStatusList.add(2);
                LambdaQueryWrapper<DmpDeviceInfo> queryWrapper1 = Wrappers.lambdaQuery();
                queryWrapper1.in(DmpDeviceInfo::getServiceStatus,serviceStatusList)
                        .eq(DmpDeviceInfo::getProductCode,list1.get(i).getProductCode())
                        .eq(DmpDeviceInfo::getDeleteFlag,0)
                        .eq(DmpDeviceInfo::getTenantId,SecurityUtils.getTenantId());
                deviceCount = dmpDeviceInfoService.count(queryWrapper1);
                responeVO.setDeviceCount(deviceCount);
                //离线告警统计
                //离线告警未处理
                int offLineUntreated = 0;
                offLineUntreated = this.count(Wrappers.<BaseAlarm>query().lambda().eq(BaseAlarm::getHandleStatus,0).eq(BaseAlarm::getAlarmType,"lx-20230700000000000").eq(BaseAlarm::getProductCode,list1.get(i).getProductCode()).eq(BaseAlarm::getTenantId,SecurityUtils.getTenantId()));
                //离线告警已处理
                int offLineProcessed = 0;
                offLineProcessed = this.count(Wrappers.<BaseAlarm>query().lambda().eq(BaseAlarm::getHandleStatus,1).eq(BaseAlarm::getAlarmType,"lx-20230700000000000").eq(BaseAlarm::getProductCode,list1.get(i).getProductCode()).eq(BaseAlarm::getTenantId,SecurityUtils.getTenantId()));
                //离线告警总数
                int offLineCount = offLineUntreated+offLineProcessed;
                //离线告警处置率
                float offLineProcessRate = 0;
                if(offLineCount != 0){
                    offLineProcessRate = ((float) offLineProcessed/offLineCount)*100;
                }
                responeVO.setOffLineUntreated(offLineUntreated);
                responeVO.setOffLineProcessed(offLineProcessed);
                responeVO.setOffLineProcessRate(offLineProcessRate);

                list.add(responeVO);
            }
        }

        int total = 0;
        if(list1.size()>0){
            total = list1.size();
        }
        int current = 0;
        Integer pageNum = baseAlarmRequestVO.getCurrent();
        Integer pageSize = baseAlarmRequestVO.getSize();
        if(pageNum != null && pageSize >0){
            current = (pageNum-1)*pageSize;
        }
        List<BaseAlarmResponeVO> records = list.stream().skip(current).limit(pageSize).collect(Collectors.toList());

        return new CommonPage<BaseAlarmResponeVO>(records,total,pageSize,pageNum);
    }

    public CommonPage<BaseAlarm> page(BaseAlarmListVO baseAlarmListVO){
        List<BaseAlarm> list = new ArrayList<>();

        LambdaQueryWrapper<BaseAlarm> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(StringUtils.isNotBlank(baseAlarmListVO.getProductCode()),BaseAlarm::getProductCode,baseAlarmListVO.getProductCode())
                .eq(StringUtils.isNotBlank(baseAlarmListVO.getAlarmType()),BaseAlarm::getAlarmType,baseAlarmListVO.getAlarmType())
                .eq(null != baseAlarmListVO.getHandleStatus(),BaseAlarm::getHandleStatus,baseAlarmListVO.getHandleStatus())
                .between(StringUtils.isNotBlank(baseAlarmListVO.getStartTime()) && StringUtils.isNotBlank(baseAlarmListVO.getEndTime()),BaseAlarm::getAlarmTime,baseAlarmListVO.getStartTime(),baseAlarmListVO.getEndTime())
                .eq(BaseAlarm::getTenantId,SecurityUtils.getTenantId())
                .orderByDesc(BaseAlarm::getId);
        List<BaseAlarm> records = this.list(queryWrapper);
        if(records.size()>0){
            for(int i=0;i<records.size();i++){
                LambdaQueryWrapper<DmpDeviceInfo> queryWrapper1 = Wrappers.lambdaQuery();
                queryWrapper1.eq(DmpDeviceInfo::getDeviceId,records.get(i).getDeviceId())
                        .eq(DmpDeviceInfo::getTenantId,SecurityUtils.getTenantId())
                        .eq(DmpDeviceInfo::getDeleteFlag,0);
                DmpDeviceInfo deviceInfo = dmpDeviceInfoService.getOne(queryWrapper1);
                if(deviceInfo != null){
                    records.get(i).setDeviceName(deviceInfo.getDeviceName());
                }

                BaseAlarmType baseAlarmType = baseMapper.getAlarmTypeOne(records.get(i).getAlarmType());
                if(baseAlarmType != null){
                    records.get(i).setTypeImg(baseAlarmType.getTypeImg());
                }

            }
        }

        if(StringUtils.isNotBlank(baseAlarmListVO.getDeviceName())){
            list = records.stream().filter(e -> baseAlarmListVO.getDeviceName().equals(e.getDeviceName())).collect(Collectors.toList());
        }else {
            if(records.size()>0){
                for(int k=0;k<records.size();k++){
                    list.add(records.get(k));
                }
            }

        }
        int total = 0;
        if(list.size()>0){
            total = list.size();
        }
        int current = 0;
        Integer pageNum = baseAlarmListVO.getCurrent();
        Integer pageSize = baseAlarmListVO.getSize();
        if(pageNum != null && pageSize >0){
            current = (pageNum-1)*pageSize;
        }
        List<BaseAlarm> info = list.stream().sorted(Comparator.comparing(BaseAlarm::getId,Comparator.reverseOrder())).skip(current).limit(pageSize).collect(Collectors.toList());

        return new CommonPage<BaseAlarm>(info,total,pageSize,pageNum);

    }

    public void update(BaseAlarm baseAlarm){
        int handleRange = baseAlarm.getHandleRange();
        if(handleRange == 0){  //批量
            LambdaQueryWrapper<BaseAlarm> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.eq(BaseAlarm::getDeviceId,baseAlarm.getDeviceId())
                    .eq(BaseAlarm::getAlarmType,baseAlarm.getAlarmType())
                    .eq(BaseAlarm::getTenantId,SecurityUtils.getTenantId())
                    .eq(BaseAlarm::getHandleStatus,0);
            List<BaseAlarm> list = this.list(queryWrapper);
            if(list.size() > 0){
                for(int i=0;i<list.size();i++){
                    list.get(i).setHandleContent(baseAlarm.getHandleContent());
                    list.get(i).setAlarmFalse(baseAlarm.getAlarmFalse());
                    list.get(i).setHandleStatus(1);
                    list.get(i).setHandleBy(SecurityUtils.getUsername());
                    list.get(i).setHandleTime(LocalDateTime.now());
                    list.get(i).setHandlePhone(SecurityUtils.getLoginUser().getSysUser().getPhonenumber());
                }
            }
            this.updateBatchById(list);

        }else {  // 单一
            baseAlarm.setHandleStatus(1);
            baseAlarm.setHandleBy(SecurityUtils.getUsername());
            baseAlarm.setHandleTime(LocalDateTime.now());
            baseAlarm.setHandlePhone(SecurityUtils.getLoginUser().getSysUser().getPhonenumber());
            this.updateById(baseAlarm);
        }
    }
    @Override
    public boolean add(BaseAlarm baseAlarm) {
        baseAlarm.setHandleStatus(0);
        if (StringUtils.isNotBlank(baseAlarm.getProductCode()) && StringUtils.isNotBlank(baseAlarm.getDeviceId()) && StringUtils.isNotBlank(baseAlarm.getAlarmType())){
            LambdaQueryWrapper<DmpProductInfo> lambdaQuery = Wrappers.lambdaQuery();
            lambdaQuery.eq(DmpProductInfo::getProductCode,baseAlarm.getProductCode())
                    .eq(DmpProductInfo::getDeleteFlag,0);
            List<DmpProductInfo> records = dmpProductInfoService.list(lambdaQuery);
            LambdaQueryWrapper<BaseFacilityDevice> lambdaQuery1 = Wrappers.lambdaQuery();
            lambdaQuery1.eq(BaseFacilityDevice::getDeviceId,baseAlarm.getDeviceId());
            List<BaseFacilityDevice> records1 = baseFacilityDeviceService.list(lambdaQuery1);
            LambdaQueryWrapper<DmpDeviceInfo> lambdaQuery2 = Wrappers.lambdaQuery();
            lambdaQuery2.eq(DmpDeviceInfo::getDeviceId,baseAlarm.getDeviceId())
                    .eq(DmpDeviceInfo::getProductCode,baseAlarm.getProductCode())
                    .eq(DmpDeviceInfo::getDeleteFlag,0);
            List<DmpDeviceInfo> records2 = dmpDeviceInfoService.list(lambdaQuery2);
            BaseAlarmType baseAlarmType = baseMapper.getAlarmTypeOne(baseAlarm.getAlarmType());
            if(records.size()>0 && baseAlarmType !=null && records2.size()>0){
                baseAlarm.setTenantId(records.get(0).getTenantId());
                if (StringUtils.isNotBlank(baseAlarm.getHandleBy()) || StringUtils.isNotBlank(baseAlarm.getHandlePhone())) {
                    baseAlarm.setHandleStatus(1);
                    LambdaQueryWrapper<BaseAlarm> queryWrapper = Wrappers.lambdaQuery();
                    queryWrapper.eq(BaseAlarm::getDeviceId, baseAlarm.getDeviceId())
                            .eq(BaseAlarm::getAlarmType, baseAlarm.getAlarmType())
                            .eq(BaseAlarm::getAlarmAddress, baseAlarm.getAlarmAddress())
                            .eq(BaseAlarm::getAlarmGrade, baseAlarm.getAlarmGrade());
                    return this.update(baseAlarm, queryWrapper);
                } else {
                    boolean saveResult = this.save(baseAlarm);
                    if (records1.size() > 0) {
                        baseAlarm.setFacilityId(records1.get(0).getFacilityId());
                    }
                    baseAlarm.setDeviceName(records2.get(0).getDeviceName());
                    baseAlarm.setTypeImg(baseAlarmType.getTypeImg());
                    baseAlarm.setAlarmType(baseAlarmType.getTypeName());
                    baseAlarm.setAlarmTime1(baseAlarm.getAlarmTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd " +
                            "HH:mm:ss")));
                    webSocket.sendMessage(baseAlarm, records.get(0).getTenantId().toString());
                    return saveResult;
                }
            }else {
                throw new BusinessException("告警数据投递失败，请检查产品编码、设备编号、告警类型，是否有误");
            }
        }else {
            throw new BusinessException("告警数据投递失败，请检查产品编码、设备编号、告警类型，不能为空");
        }
    }
}













