package com.usky.iot.service.impl;



import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.UUIDUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.domain.*;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.mapper.DmpDeviceInfoMapper;
import com.usky.iot.service.BaseFacilityDeviceService;
import com.usky.iot.service.DmpDeviceInfoService;
import com.usky.iot.service.DmpDeviceStatusService;
import com.usky.iot.service.vo.*;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备信息表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-10-08
 */
@Service
public class DmpDeviceInfoServiceImpl extends AbstractCrudService<DmpDeviceInfoMapper, DmpDeviceInfo> implements DmpDeviceInfoService {
    @Autowired
    private DmpDeviceStatusService dmpDeviceStatusService;

    @Autowired
    private BaseFacilityDeviceService baseFacilityDeviceService;

    @Override
    public void add(DmpDeviceInfo dmpDeviceInfo) {
        if (checkNameUnique(dmpDeviceInfo)){
            throw new BusinessException("新增设备信息'" + dmpDeviceInfo.getDeviceId() + "'失败，设备信息已存在");
        }
        List<DmpProductInfo> list = baseMapper.getProductInfo(dmpDeviceInfo.getProductCode());
        if(CollectionUtils.isNotEmpty(list)){
            dmpDeviceInfo.setDeviceType(list.get(0).getDeviceType());
            dmpDeviceInfo.setProductId(list.get(0).getId());
            dmpDeviceInfo.setCreatedBy(SecurityUtils.getUsername());
            dmpDeviceInfo.setCreatedTime(LocalDateTime.now());
            dmpDeviceInfo.setTenantId(SecurityUtils.getTenantId());
            dmpDeviceInfo.setServiceStatus(1);
            this.save(dmpDeviceInfo);

            DmpDeviceStatus dmpDeviceStatus = new DmpDeviceStatus();
            dmpDeviceStatus.setDeviceId(dmpDeviceInfo.getDeviceId());
            dmpDeviceStatus.setProductId(dmpDeviceInfo.getProductId());
            dmpDeviceStatus.setDeviceStatus(2);
            dmpDeviceStatus.setLastOfflineTime(LocalDateTime.now());
            dmpDeviceStatusService.save(dmpDeviceStatus);
        }else{
            throw new BusinessException("新增设备信息'" + dmpDeviceInfo.getDeviceId() + "'失败，无对应产品信息");
        }
    }

    @Override
    public void update(DmpDeviceInfo dmpDeviceInfo) {
        dmpDeviceInfo.setUpdatedBy(SecurityUtils.getUsername());
        dmpDeviceInfo.setUpdatedTime(LocalDateTime.now());
        if (checkNameUnique(dmpDeviceInfo)){
            throw new BusinessException("修改设备信息'" + dmpDeviceInfo.getDeviceId() + "'失败，设备信息已存在");
        }
        this.updateById(dmpDeviceInfo);

    }

    @Override
    public boolean checkNameUnique(DmpDeviceInfo dmpDeviceInfo) {
        Integer id = null == dmpDeviceInfo.getId() ? -1 : dmpDeviceInfo.getId();
        LambdaQueryWrapper<DmpDeviceInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(DmpDeviceInfo::getDeviceId,dmpDeviceInfo.getDeviceId())
                .eq(DmpDeviceInfo::getProductCode,dmpDeviceInfo.getProductCode())
                .eq(DmpDeviceInfo::getDeleteFlag,0)
                .eq(DmpDeviceInfo::getTenantId,SecurityUtils.getTenantId());
        DmpDeviceInfo one = this.getOne(queryWrapper);
        return null != one && !Objects.equals(one.getId(), id);
    }

    @Override
    public boolean checkNameUnique1(DmpDeviceInfo dmpDeviceInfo,Integer tenantId) {
        Integer id = null == dmpDeviceInfo.getId() ? -1 : dmpDeviceInfo.getId();
        LambdaQueryWrapper<DmpDeviceInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(DmpDeviceInfo::getDeviceId,dmpDeviceInfo.getDeviceId())
                .eq(DmpDeviceInfo::getProductCode,dmpDeviceInfo.getProductCode())
                .eq(DmpDeviceInfo::getDeleteFlag,0)
                .eq(DmpDeviceInfo::getTenantId,tenantId);
        DmpDeviceInfo one = this.getOne(queryWrapper);
        return null != one && !Objects.equals(one.getId(), id);
    }

    @Override
    public CommonPage<DmpDeviceInfo> page(DmpDeviceInfoRequest diRequest) {
        List<DmpDeviceInfo> list = new ArrayList<>();
        LambdaQueryWrapper<DmpDeviceInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(StringUtils.isNotBlank(diRequest.getDeviceName()),DmpDeviceInfo::getDeviceName,diRequest.getDeviceName())
                .eq(diRequest.getProductId() != null,DmpDeviceInfo::getProductId,diRequest.getProductId())
                .eq(diRequest.getServiceStatus() != null,DmpDeviceInfo::getServiceStatus,diRequest.getServiceStatus())
                .eq(DmpDeviceInfo::getDeleteFlag,0)
                .eq(DmpDeviceInfo::getTenantId,SecurityUtils.getTenantId())
                .orderByDesc(DmpDeviceInfo::getId);
        List<DmpDeviceInfo> list1 = this.list(queryWrapper);

        if(CollectionUtils.isNotEmpty(list1)){
            List<String> devList = new ArrayList<>();
            for(int i=0;i<list1.size();i++){
                devList.add(list1.get(i).getDeviceId());
            }

            if(devList.size() > 0){
                LambdaQueryWrapper<DmpDeviceStatus> queryWrapper1 = Wrappers.lambdaQuery();
                queryWrapper1.in(DmpDeviceStatus::getDeviceId,devList);
                List<DmpDeviceStatus> statusList = dmpDeviceStatusService.list(queryWrapper1);
                if(statusList.size() > 0){
                    for(int i=0;i<list1.size();i++){
                        for(int j=0;j<statusList.size();j++){
                            if(list1.get(i).getProductId().equals(statusList.get(j).getProductId()) && list1.get(i).getDeviceId().equals(statusList.get(j).getDeviceId())){
                                list1.get(i).setDeviceStatus(statusList.get(j).getDeviceStatus());
                                continue;
                            }
                        }
                    }
                }
            }


            if(diRequest.getDeviceStatus() != null){
                list = list1.stream().filter(m-> m.getDeviceStatus().equals(diRequest.getDeviceStatus())).collect(Collectors.toList());
            }else{
                for(int j=0;j<list1.size();j++){
                    list.add(list1.get(j));
                }
            }
        }

        Integer total = 0;
        if(list.size() > 0){
            total = list.size();
        }

        Integer current = 0;
        Integer pageCurrent = diRequest.getCurrent();
        Integer pageSize = diRequest.getSize();
        if(pageCurrent != null && pageSize > 0){
            current = (pageCurrent-1)*pageSize;
        }

        List<DmpDeviceInfo> records = list.stream().skip(current).limit(pageSize).collect(Collectors.toList());

        return new CommonPage<>(records,total,pageSize,pageCurrent);
    }

    @Override
    public boolean remove(Integer id) {
        DmpDeviceInfo dmpDeviceInfo = this.getById(id);
        Optional.ofNullable(dmpDeviceInfo).orElseThrow(() -> new BusinessException("设备不存在"));

        //已关联设施的不能删除
        LambdaQueryWrapper<BaseFacilityDevice> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(BaseFacilityDevice::getDeviceId,dmpDeviceInfo.getDeviceId());
        int count = baseFacilityDeviceService.count(queryWrapper);
        if(count > 0){
            throw new BusinessException("已关联设施的不能删除");
        }

        dmpDeviceInfo.setDeleteFlag(1);
        return this.updateById(dmpDeviceInfo);
    }

    @Override
    public List<Map<String, Object>> deviceCollect(List<Integer> productIds) {
        QueryWrapper<DmpDeviceInfo> query = Wrappers.query();
        query.select("product_id as productId","count(*) as count","count(service_status != 1 or null) as serviceCount")
                .in(CollectionUtil.isNotEmpty(productIds),"product_id",productIds)
                .eq("delete_flag",0)
                .eq("tenant_id",SecurityUtils.getTenantId())
                .groupBy("product_id");
        return this.listMaps(query);
    }

    @Override
    public int deviceCountByProductId(Integer productId) {
        LambdaQueryWrapper<DmpDeviceInfo> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(DmpDeviceInfo::getProductId,productId)
                .eq(DmpDeviceInfo::getDeleteFlag,0);
        return this.count(lambdaQuery);
    }

    @Override
    public void addByFile(MultipartFile multipartFiles) {
        ImportParams params = new ImportParams();
        params.setHeadRows(1);
        params.setTitleRows(1);
        String err="文件导入失败";
        try {
            List<DeviceImportVo> deviceImportVos = ExcelImportUtil.importExcel(multipartFiles.getInputStream(),
                    DeviceImportVo.class, params);
            if (CollectionUtils.isNotEmpty(deviceImportVos)){
                int rot=0;
                for (DeviceImportVo deviceImportVo:deviceImportVos) {
                    DmpDeviceInfo dmpDeviceInfo = BeanMapperUtils.map(deviceImportVo, DmpDeviceInfo.class);
                    if (StringUtils.isBlank(dmpDeviceInfo.getDeviceId())||StringUtils.isBlank(dmpDeviceInfo.getProductCode())||StringUtils.isBlank(dmpDeviceInfo.getDeviceName())||StringUtils.isBlank(dmpDeviceInfo.getInstallAddress())){
                        int h=rot+3;
                        err="文件导入失败，第"+h+"行数据导入失败,必填字段不能为空";
                        throw  new BusinessException(err);
                    }
                    try{
                        this.add(dmpDeviceInfo);
                    }catch (Exception e){
                        int h=rot+3;
                        err="文件导入失败，第"+h+"行数据导入失败";
                        throw  new BusinessException(err);
                    }
                    rot++;
                }
            }else {
                err="文件不能为空";
                throw new BusinessException(err);
            }
        }catch (Exception e){
            throw  new BusinessException(err);
        }
    }

    @Override
    public void importDeviceInfo(List<DmpDeviceInfo> devList){
        List<DmpDeviceStatus> dmpDeviceStatusList = new ArrayList<>();
        if(CollectionUtils.isEmpty(devList)){
            throw new BusinessException("数据不能为空");
        }
        for(int i=0;i<devList.size();i++){
            List<DmpProductInfo> list = baseMapper.getProductInfo(devList.get(i).getProductCode());
            if(CollectionUtils.isNotEmpty(list)){

                if (checkNameUnique1(devList.get(i),list.get(0).getTenantId())){
                    throw new BusinessException("新增设备信息'" + devList.get(i).getDeviceId() + "'失败，设备信息已存在");
                }

                devList.get(i).setDeviceType(list.get(0).getDeviceType());
                devList.get(i).setProductId(list.get(0).getId());
                devList.get(i).setCreatedBy(SecurityUtils.getUsername());
                devList.get(i).setCreatedTime(LocalDateTime.now());
                devList.get(i).setTenantId(list.get(0).getTenantId());
                devList.get(i).setServiceStatus(1);

                DmpDeviceStatus dmpDeviceStatus = new DmpDeviceStatus();
                dmpDeviceStatus.setDeviceId(devList.get(i).getDeviceId());
                dmpDeviceStatus.setProductId(devList.get(i).getProductId());
                dmpDeviceStatus.setDeviceStatus(2);
                dmpDeviceStatus.setLastOfflineTime(LocalDateTime.now());
                dmpDeviceStatusList.add(dmpDeviceStatus);
            }else{
                throw new BusinessException("新增设备信息'" + devList.get(i).getDeviceId() + "'失败，无对应产品信息");
            }
        }
        this.saveBatch(devList);
        dmpDeviceStatusService.saveBatch(dmpDeviceStatusList);
    }

    @Override
    public void export(DmpDeviceInfoRequest dmpDeviceInfoRequest, HttpServletResponse response) {
        Workbook workbook = null;
        try {
            ExportParams params = new ExportParams(null, "设备列表");
            workbook = ExcelExportUtil.exportBigExcel(params, DmpDeviceExcelVO.class,
                    (o, i) -> {
                        dmpDeviceInfoRequest.setCurrent(i);
                        dmpDeviceInfoRequest.setSize(30);
                        CommonPage<DmpDeviceInfo> list = this.page(dmpDeviceInfoRequest);
                        return new ArrayList<>(BeanMapperUtils.mapList(list.getRecords(),DmpDeviceInfo.class,DmpDeviceExcelVO.class));
                    },null);
            if (null != workbook) {
                exportExcel(workbook,response);
            } else {
                throw new BusinessException("表格数据为空");
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            throw new BusinessException("导出文件失败"+e.getMessage());
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
    }

    public void exportExcel(Workbook workbook, HttpServletResponse response){
        //定义一个流，用作流式输出
        ServletOutputStream outputStream = null;
        try {
            //使用流的形式传输
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            //防止中文乱码
            response.setCharacterEncoding("utf-8");
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if(null!=outputStream){
                try {
                    //关闭流
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * 获取表格数据
     * @param multipartFile
     * @return
     */
    public List<DmpDeviceInfo> getDeviceByFile(Integer productId,MultipartFile multipartFile){
        ImportParams importParams = new ImportParams();
        importParams.setHeadRows(1);
        try {
            List<DmpDeviceExportVO> dmpDeviceExportVOS = ExcelImportUtil.importExcel(
                    multipartFile.getInputStream(),
                    DmpDeviceExportVO.class,
                    importParams
            );
            if (CollectionUtils.isEmpty(dmpDeviceExportVOS)){
                throw new BusinessException("表格数据为空");
            }
            List<DmpDeviceInfo> deviceInfos = BeanMapperUtils.mapList(dmpDeviceExportVOS, DmpDeviceExportVO.class, DmpDeviceInfo.class);
            long count = deviceInfos.stream()
                    .peek(device -> {device.setProductId(productId);
                    device.setDeviceId(UUIDUtils.uuid());})
                    .filter(device -> StringUtils.isBlank(device.getDeviceName()) ||
                    StringUtils.isBlank(device.getDeviceName())
                    || StringUtils.isBlank(device.getSimCode())
                    || null == device.getSubscribeFlag()
            ).count();
            long count1 = deviceInfos.stream()
                    .collect(Collectors.toMap(DmpDeviceInfo::getDeviceName, s -> 1, Integer::sum))
                    .entrySet()
                    .stream()
                    .filter(entry -> entry.getValue() > 1)
                    .count();
            if (count > 0){
                throw new BusinessException("表格字段不能为空");
            }
            if (count1 > 0){
                throw new BusinessException("设备名称不能重复");
            }
            return deviceInfos;
        }catch (Exception e){
            throw new BusinessException("解析表格异常,检查表格格式"+e.getMessage());
        }
    }
}
